package com.soss.system.service.impl;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.soss.common.constant.HttpStatus;
import com.soss.common.core.domain.AjaxResult;
import com.soss.common.core.domain.model.LoginUser;
import com.soss.common.exception.ServiceException;
import com.soss.common.utils.StringUtils;
import com.soss.common.utils.spring.SpringUtils;
import com.soss.system.constants.OrderStatusConstant;
import com.soss.system.domain.*;
import com.soss.system.jiguang.impl.JiGuangPushServiceImpl;
import com.soss.system.service.IOrderService;
import com.soss.system.domain.vo.OrderQuery;
import com.soss.system.mapper.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.support.atomic.RedisAtomicInteger;
import org.springframework.security.core.token.TokenService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

/**
 * 订单Service业务层处理
 * 
 * @author zxq
 * @date 2022-04-28
 */
@Service
public class OrderServiceImpl implements IOrderService
{
    @Autowired
    private OrderMapper orderMapper;
    @Autowired
    private ShopGoodsSkuMapper shopGoodsSkuMapper;
    @Autowired
    private OrderDetailMapper orderDetailMapper;
    @Autowired
    private GoodsMapper goodsMapper;
    @Autowired
    private MachineMapper machineMapper;
    @Autowired
    private MachineApiServiceImpl machineApiService;
    @Autowired
    private ShopMapper shopMapper;
    @Autowired
    private OrderOperationLogServiceImpl orderOperationLogService;
    @Autowired
    private OrderSnapshotServiceImpl orderSnapshotService;
    @Autowired
    private JiGuangPushServiceImpl jiGuangPushService;



    /**
     * 查询订单
     * 
     * @param id 订单主键
     * @return 订单
     */
    @Override
    public Order selectOrderById(String id)
    {
        Order order = orderMapper.selectOrderById(id);
        order.setShop(shopMapper.selectShopById(order.getShopId()));
        OrderSnapshot orderSnapshot = orderSnapshotService.selectOrderSnapshotByOrderId(order.getId());
        String snapshot = orderSnapshot.getSnapshot();
        List<OrderDetail> orderDetails1 = JSONObject.parseArray(snapshot, OrderDetail.class);
        order.setOrderDetails(orderDetails1);
        return order;
    }

    /**
     * 查询订单列表
     * 
     * @param order 订单
     * @return 订单
     */
    @Override
    public List<Order> selectOrderList(OrderQuery order)
    {
        List<Order> orders = orderMapper.selectOrderByQuery(order);
        for (Order order1 : orders) {
            order1.setShop(shopMapper.selectShopById(order1.getShopId()));
        }
        return orders;
    }

    /**
     * 新增订单
     * 
     *
     * @param loginUser
     * @param order 订单
     * @return 结果
     */
    @Override
    @Transactional
    public AjaxResult insertOrder(LoginUser loginUser, Order order)
    {
        AjaxResult result = check(order);
        if(HttpStatus.SUCCESS != (int)result.get(result.CODE_TAG)){
            return result;
        }
        order.setState(OrderStatusConstant.Unpaid);
        order.setUserId(loginUser.getOpenId());
        order.setUserName(loginUser.getUsername());
        order.setCreatedAt(new Date());
        order.setOrderNum(getOrderNum(order.getShopId()));
        order.setOrderNo(getOrderNo(order.getSource()));
        order.setUpdatedAt(new Date());
        String machineId = null;
        if(StringUtils.isEmpty(order.getMachineId())){
            Machine machine =new Machine();
            machine.setShopId(Long.parseLong(order.getShopId()));
            List<Machine> machines = machineMapper.selectMachineList(machine);
            if(machines!=null &&!machines.isEmpty()){
                Machine machine1 = machines.get(0);
                if(!"1".equals(machine1.getState())){
                    return AjaxResult.error("机器暂时不可用");
                }
                machineId = machine1.getId();
                order.setMachineId(machineId);
            }else{
                return AjaxResult.error("该店铺暂无绑定机器");
            }
        }
        int  i = orderMapper.insertOrder(order);
        List<OrderDetail> orderDetails = order.getOrderDetails();
        if(orderDetails !=null && !orderDetails.isEmpty()){
            for (OrderDetail orderDetail : orderDetails) {
                orderDetail.setOrderId(order.getId());
                orderDetail.setCreatedAt(new Date());
                orderDetail.setUpdatedAt(new Date());
                orderDetail.setMachineId(machineId);
                orderDetailMapper.insertOrderDetail(orderDetail);
            }
        }
        //发送数据给机器
        machineApiService.createOrder(order);
        orderOperationLogService.insertOrderOperationLog(order);
        orderSnapshotService.saveOrderSnapshot(order);
        return AjaxResult.success(order.getOrderNo());

    }

    /**
     * 修改订单
     * 
     * @param order 订单
     * @return 结果
     */
    @Override
    public int updateOrder(Order order)
    {
        return orderMapper.updateOrder(order);
    }

    /**
     * 批量删除订单
     * 
     * @param ids 需要删除的订单主键
     * @return 结果
     */
    @Override
    public int deleteOrderByIds(String[] ids)
    {
        return orderMapper.deleteOrderByIds(ids);
    }

    /**
     * 删除订单信息
     * 
     * @param id 订单主键
     * @return 结果
     */
    @Override
    public int deleteOrderById(String id)
    {
        return orderMapper.deleteOrderById(id);
    }

    @Override
    public AjaxResult check(Order order) {
        //验证订单状态
        List<OrderDetail> orderDetails = order.getOrderDetails();
        if(orderDetails==null ||orderDetails.isEmpty()){
            return AjaxResult.error("订单信息不能为空");
        }

        for (OrderDetail orderDetail : orderDetails) {
            ShopGoodsSku shopGoodsSku = new ShopGoodsSku();
            shopGoodsSku.setSkuId(Long.parseLong(orderDetail.getSkuId()));
            shopGoodsSku.setShopId(Long.valueOf(order.getShopId()));
            shopGoodsSku.setIsDeleted("0");
            List<ShopGoodsSku> shopGoodsSkus = shopGoodsSkuMapper.selectShopGoodsSkuList(shopGoodsSku);
            if(shopGoodsSkus ==null ||shopGoodsSkus.isEmpty()){
                return AjaxResult.error("该商品已不存在："+orderDetail.getGoodsName());
            }
            ShopGoodsSku shopGoodsSku1 = shopGoodsSkus.get(0);
            if(!"1".equals(shopGoodsSku1.getState())){
                return AjaxResult.error("该商品所选种类已经告罄，请重新进行选择："+orderDetail.getGoodsName());
            }
        }
        return AjaxResult.success();
    }

    @Override
    public List<Order> getMyOrder(String openId) {

        List<Order> orders = orderMapper.selectByUserId(openId);
        for (Order order : orders) {
            order.setShop(shopMapper.selectShopById(order.getShopId()));
            OrderSnapshot orderSnapshot = orderSnapshotService.selectOrderSnapshotByOrderId(order.getId());
            order.setOrderDetails(JSONArray.parseArray(orderSnapshot.getSnapshot(),OrderDetail.class));
        }
        return orders;
    }

    @Override
    @Transactional
    public int cancel(String orderId) {
        Order order = orderMapper.selectOrderById(orderId);
        if(OrderStatusConstant.Unpaid.equals(order.getState())){
            order.setState("8");
            order.setUpdatedAt(new Date());
            orderMapper.updateOrder(order);
            orderOperationLogService.insertOrderOperationLog(order);
            machineApiService.updateOrder(order);
        }else if(OrderStatusConstant.Paid.equals(order.getState())){
            order.setState("9");
            order.setUpdatedAt(new Date());
            orderMapper.updateOrder(order);
            orderOperationLogService.insertOrderOperationLog(order);
            machineApiService.updateOrder(order);
            jiGuangPushService.pushOrderState(order);
        }else{
            throw new ServiceException("该订单不允许取消");
        }

        return 0;
    }



    /**
     * 获取机器顺序号
     * @param shopId
     * @return
     */
    private String getOrderNum(String shopId){
        Calendar calendar = Calendar.getInstance();
        String key = calendar.getTime().toString().substring(0,3);
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
        String format = simpleDateFormat.format(new Date());
        String redisKey = "shopId"+format+shopId;
        RedisAtomicInteger redisAtomicInteger = new RedisAtomicInteger(redisKey,SpringUtils.getBean(StringRedisTemplate.class).getConnectionFactory());
        if(0 == redisAtomicInteger.get()){
            redisAtomicInteger.set(30);
        }
        redisAtomicInteger.expire(1, TimeUnit.DAYS);
        int andIncrement = redisAtomicInteger.getAndIncrement();
        return key+String.format("%03d",andIncrement);
    }
    //获取订单号
    private String getOrderNo(Integer source){
        String channel = "M";
        if(source !=null){
            if(source == 1){
                channel="A";
            }else if(source ==2){
                channel="B";
            }
        }
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyyMMddHH");
        String format = simpleDateFormat.format(new Date());
        String key = "orderNo";
        RedisAtomicInteger redisAtomicInteger = new RedisAtomicInteger(key,SpringUtils.getBean(StringRedisTemplate.class).getConnectionFactory());
        int andIncrement = redisAtomicInteger.getAndIncrement();
        if(andIncrement>9999){
            redisAtomicInteger.set(0);
            andIncrement = 0;
        }
        String format1 = String.format("%04d", andIncrement);
        int randomNumber = new Random().nextInt(99);
        return "N"+format+channel+format1+randomNumber;
    }

    public  List<JSONObject>  getOrderInfo(String machineCode) {
        Machine machine = machineMapper.selectMachineByCode(machineCode);
        List<String> status = Arrays.asList("3","4","5");
        List<Order> orders = orderMapper.selectOrder(status, machine.getId());
        Map<String, List<Order>> collect = orders.stream().collect(Collectors.groupingBy(Order::getState));
        List<JSONObject> objects = new ArrayList<>();
        for(String key:collect.keySet()){
            JSONObject jsonObject =new JSONObject();
            objects.add(jsonObject);
            jsonObject.put("state",key);
            List<Order> orders1 = collect.get(key);
            List<JSONObject> jsonObjects = new ArrayList<>();
            jsonObject.put("orders",jsonObjects);
            for (Order order : orders1) {
                JSONObject jsonObject1 = new JSONObject();
                jsonObjects.add(jsonObject1);
                jsonObject1.put("orderId",order.getId());
                jsonObject1.put("orderNum",order.getOrderNum());
                jsonObject1.put("orderNo",order.getOrderNo());
            }
        }
        return objects;

    }


    public String refundOrder(String orderId) {
        Order order = orderMapper.selectOrderById(orderId);
        List<String> status = Arrays.asList("2","3");
        if(status.contains(order.getState())){
            //todo 创建退款单，进行退款
        }else{
            //todo 创建退款单

        }
        return null;
    }

    public Map<String,String> getMyFristOrder(String openId) {
        List<String> status = Arrays.asList("2","3","4");
        Order order = orderMapper.selectHomeByUserId(openId,status);
        if(order ==null){
            return null;
        }
        OrderDetail orderDetail = new OrderDetail();
        orderDetail.setOrderId(order.getId());
        List<OrderDetail> orderDetails = orderDetailMapper.selectOrderDetailList(orderDetail);
        order.setOrderDetails(orderDetails);
        String waitTime = getWaitTime(order);
        Map<String,String> map =new HashMap<>();
        map.put("waitTime",waitTime);
        map.put("pickCode",order.getPickCode());
        map.put("orderNum",order.getOrderNum());
        return map;

    }

    public String getWaitTime(Order order) {
        String machineId = order.getMachineId();
        List<String> status = Arrays.asList("2","3");
        List<Order> orders = orderMapper.selectOrder(status, machineId);
        List<OrderDetail> list = new ArrayList<>();
        list.addAll(order.getOrderDetails());
        for (Order order1 : orders) {
            OrderDetail orderDetail = new OrderDetail();
            orderDetail.setOrderId(order1.getId());
            List<OrderDetail> orderDetails = orderDetailMapper.selectOrderDetailList(orderDetail);
            if(CollectionUtils.isEmpty(orderDetails)){

            }else{
                list.addAll(orderDetails);
            }
        }
        Long takeTimeCount =0L;
        for (OrderDetail orderDetail : list) {
            Long takeTime = goodsMapper.selectGoodsById(orderDetail.getGoodsId()).getTakeTime();
            takeTimeCount+=takeTime;
        }

        return  String.valueOf((int)(takeTimeCount/60));

    }
}
